import excel
import operations
import json

# ---------------------------------------------------------------- FUNCTIONS ----------------------------------------------------------------
def equals(json1,json2,keys_to_compare):
    for key in keys_to_compare:
        if not(key in json1 and key in json2):
            return False
        if (json1[key] != json2[key]):
            return False
    return True

def removeFromFirstPresenceArray(element,array_json_target,keys_to_compare:list[str],inOrder=True):
    for i,json_obj in enumerate(array_json_target) :
        if equals(json_obj,element,keys_to_compare):
            return array_json_target[i+1:]
    return array_json_target

def createDefaultFiles(filename_excel, sheet_name):
    excel.createExcelIfNotExisting(filename_excel)
    excel.createSheetIfNotExisting(filename_excel,sheet_name)
# ---------------------------------------------------------------- MAIN ----------------------------------------------------------------

def main_function(filename_excel, sheet_name):
    # Ordre
    # 1 : Fetch excel vers JSON
    # 2 : Fetch web vers JSON
    # 3 : Ajuster web en fonction de plus récent excel
    # 4 : Ajouter à excel web ajusté
    
    # 1
    excel_json_str = excel.excelToJson(filename_excel, sheet_name)
    excel_json = json.loads(excel_json_str)
    print("JSON generated from xlsx file")

    # 2
    web_json_str = operations.fetchOperationsToJson()
    web_json = json.loads(web_json_str)
    print("JSON generated from web Société Générale")

    # 3
    if len(excel_json) > 0 : web_json_new = removeFromFirstPresenceArray(excel_json[-1],web_json,['date','debit','credit','nature'])
    else : web_json_new = web_json

    #4
    if len(web_json_new) > 0 : excel.appendJsonToExcel(web_json_new,filename_excel,sheet_name)

    print(f"{len(web_json_new)} lines added to excel")

def isUserIDRightFormat(user_id):
    try :
        if len(str(int(user_id))) != 8 :
            return False
    except :
        return False


import click
import os
from functions import main_function,isUserIDRightFormat

sheet_name = 'account_operations'

@click.command()
@click.option('--set-user-id', '-u',show_default="",default=None, help="Will set the user id for Société Générale login")
@click.argument('filename',nargs=1)
def cli(set_user_id,filename):

    if os.getenv("USER_ID") == None :
        click.echo("USER_ID not found, please use '-u *your user_id*' flag to initiate it")

    #CLIENT
    if set_user_id != None:
        if isUserIDRightFormat(set_user_id):
            click.echo(f"Setting the USER_ID variable to {set_user_id}")
            os.environ["USER_ID"] = set_user_id
        else :
            return click.echo("Wrong format of user_id parameter : must be 8 digits (ex: 12345678)")
    
    if not(filename == "" or len(filename.split("."))<=1 or filename.split(".")[0]=="" or filename.split(".")[1]!='xlsx'):
        main_function(filename,sheet_name)
    else :
        click.echo(f"Incorrect filename : '{filename}', filename format must be like 'exemple.xlsx'")


import os
from openpyxl import load_workbook
import pandas as pd
import json

# ------------------------------------------------- MANAGEMENT FUNCTIONS ----------------------------------------------------

def createExcelIfNotExisting(filename_excel)->bool:
    if not(os.path.isfile(filename_excel)):
        print(f"Excel file '{filename_excel}' doesn't exists, creating a new file...")
        with pd.ExcelWriter(filename_excel,engine='xlsxwriter'):
            pass
        return False
    return True

def createSheetIfNotExisting(filename_excel, sheet_name)->bool:
    wb_readonly = load_workbook(filename_excel, read_only=True)
    if not(sheet_name in wb_readonly.sheetnames) :
        wb_readonly.close()
        print(f"Sheet '{sheet_name}' in '{filename_excel}' doesn't exists, creating a new sheet")
        wb = load_workbook(filename_excel)
        wb.create_sheet(sheet_name)
        wb.save(filename_excel)
        wb.close()
        return False
    return True

#------------------------------------------------------ JSON/EXCEL FUNCTIONS ----------------------------------------------------

def appendJsonToExcel(data,filename_excel,sheet_name):
    # Json to Dataframe
    df = pd.json_normalize(data)

    # Create excel file if not existing
    excelExist = createExcelIfNotExisting(filename_excel)

    # Create sheet if not existing
    sheetExist = createSheetIfNotExisting(filename_excel,sheet_name)

    # Append to Excel File if existing:
    if sheetExist&excelExist:
        with pd.ExcelWriter(filename_excel,mode='a',if_sheet_exists='overlay',engine='openpyxl') as writer:
            df.to_excel(writer, sheet_name=sheet_name,startrow=writer.sheets[sheet_name].max_row,header=None,index=None)
    else :
        with pd.ExcelWriter(filename_excel,engine='openpyxl') as writer:
            df.to_excel(writer, sheet_name=sheet_name,index=None)

def excelToJson(filename_excel,sheet_name,save_in_file:str=None):
    # Get data from .xlsx file
    try :
        excel_data_df = pd.read_excel(filename_excel, sheet_name=sheet_name,header=0)

        json_str = excel_data_df.to_json(orient="records")
        json_dict = json.loads(json_str)

        if (save_in_file != None):
            with open(save_in_file,mode='w') as f:
                json.dump(json_dict,f,indent=4)

        return json.dumps(json_dict, indent=4)
    except :
        return json.dumps([])


from setuptools import setup,find_packages
from pip._internal.req import parse_requirements

#Actual setup
setup(
    name="asteq-treso",
    version='0.1',
    description='Automation of the Accountability for the BDS of IMT Atlantique Nantes',
    long_description="Scrapping + Save new transactions",
    long_description_content_type='text/plain',
    license='MIT',
    author="gloxounet",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/Gloxounet/asteq-treso',
    keywords='asteq',
    py_modules=['excel','operations','functions','cli'],
    install_requires=[
        'selenium',
        'pandas',
        'openpyxl',
        'Click',
        'webdriver_manager',
        'packaging',
        'xlsxwriter'
        ],
    python_requires='>=3',
    entry_points='''
        [console_scripts]
        asteq-treso=cli:cli
    ''',
)


# ----------------------------------------- IMPORTS -------------------------------------------------------
# Env variables
from dataclasses import dataclass,asdict
import os

import json

# Selenium 4 : Minimal imports
from selenium import webdriver
from selenium.webdriver.chrome.service import Service as ChromeService
from webdriver_manager.chrome import ChromeDriverManager

# Selenium locators
from selenium.webdriver.common.by import By

# Selenium Waits
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

# Selenium no logs
from selenium.webdriver.chrome.options import Options

chrome_options = Options()
chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])

# ----------------------------------------- VARIABLES -------------------------------------------------------

# Global variables
soge_path = "https://professionnels.societegenerale.fr/icd-web/syd-front/index-comptes.html#comptes"

# ----------------------------------------- CLASSES ---------------------------------------------------------

@dataclass
class Operation :
    date: str
    date_value: str
    nature: str
    debit: float
    credit: float
    solde_comptable:float

# ----------------------------------------- FUNCTIONS -------------------------------------------------------

def elementPresentCheck(driver, byWhat, path:str,callback=lambda x:x)->bool:
    try:
        element = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((byWhat, path))
            )
        callback(element)
        return True
    except Exception as e:
        print(e)
        return False

def cookiePopUpCheck(driver)->bool:
    def callback(element):
        button = element.find_element(By.ID,"popin_tc_privacy_button_3")
        button.click()

    return elementPresentCheck(driver, By.ID, path="popin_tc_privacy",callback=callback)

def userInputCheck(driver,user_id:str=None):
    def callback(element):
        if user_id is not None :
            element.send_keys(user_id)

    return elementPresentCheck(driver,By.ID,path="user_id",callback=callback)

def operationArrayToJson(array:list[Operation])-> str:
    d = list(map(lambda x:asdict(x),array))
    return json.dumps(d,indent=4)
        
def betterParseInt(x:str)->float:
    string = x.replace(" ","").replace(",",".")
    if string == "" :
        return None
    return float(string)

# ----------------------------------------- MAIN -------------------------------------------------------


def fetchOperationsToJson():
    # Create driver instance
    driver = webdriver.Chrome(service=ChromeService(ChromeDriverManager().install()),options=chrome_options)
    driver.get(soge_path)

    # Check if Sogé asking for cookies and accept if then
    cookiePopUpCheck(driver)

    # Fill user_id field
    userInputCheck(driver, os.getenv('USER_ID'))

    print("Press Enter when you are ready to scrap")
    input()

    table = driver.find_element(By.ID,"idVirementList")
    releves = table.find_elements(By.CLASS_NAME,"button")

    releves_final = []
    for releve in releves:
        categories = releve.find_elements(By.TAG_NAME,"td")
        operation = Operation(
            date=categories[0].text,
            date_value=categories[1].text,
            nature=categories[2].text,
            debit=betterParseInt(categories[3].text),
            credit=betterParseInt(categories[4].text),
            solde_comptable=betterParseInt(categories[5].text),
        )
        releves_final.append(operation)
    releves_final.reverse()
    return operationArrayToJson(releves_final)